"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeneratedComposite = void 0;
const ern_core_1 = require("ern-core");
const generateComposite_1 = require("./generateComposite");
const path_1 = __importDefault(require("path"));
class GeneratedComposite {
    static async generate(config) {
        await (0, generateComposite_1.generateComposite)(config);
        return new GeneratedComposite(config);
    }
    constructor(config) {
        this.config = config;
        this.path = config.outDir;
        this.packageJson = (0, ern_core_1.readPackageJsonSync)(config.outDir);
    }
    async getJsApiImpls() {
        var _a;
        return Promise.resolve((_a = this.config.jsApiImplDependencies) !== null && _a !== void 0 ? _a : []);
    }
    async getMiniApps() {
        const miniAppsPackages = await this.getMiniAppsPackages();
        return miniAppsPackages.map((p) => new ern_core_1.BaseMiniApp({ miniAppPath: p.path, packagePath: p.packagePath }));
    }
    async getResolvedNativeDependencies() {
        const nativeDependencies = await this.getNativeDependencies();
        return ern_core_1.nativeDepenciesVersionResolution.resolveNativeDependenciesVersionsEx(nativeDependencies);
    }
    async getInjectableNativeDependencies(platform) {
        const dependencies = await this.getResolvedNativeDependencies();
        const result = [];
        for (const dependency of dependencies.resolved) {
            // Always include react-native
            if (dependency.name === 'react-native') {
                result.push(dependency);
                continue;
            }
            if (platform === 'android') {
                if (await ern_core_1.manifest.getPluginConfig(dependency, 'android')) {
                    result.push(dependency);
                }
            }
            else {
                if (await ern_core_1.manifest.getPluginConfig(dependency, 'ios')) {
                    result.push(dependency);
                }
            }
        }
        return result;
    }
    /**
     * Get the package name of the MiniApps present in this Composite
     */
    async getMiniAppsPackages() {
        const result = [];
        for (const key of Object.keys(this.packageJson.dependencies)) {
            const ppValue = ern_core_1.PackagePath.fromString(this.packageJson.dependencies[key]);
            const ppKey = ern_core_1.PackagePath.fromString(key);
            if (!ppValue.isFilePath &&
                this.config.miniApps.some((p) => p.basePath === ppValue.basePath)) {
                result.push({
                    name: key,
                    packagePath: ppValue,
                    path: path_1.default.join(this.path, 'node_modules', key),
                });
            }
            else if (this.config.miniApps.some((p) => p.basePath === ppKey.basePath)) {
                result.push({
                    name: key,
                    packagePath: ern_core_1.PackagePath.fromString(`${key}@${this.packageJson.dependencies[key]}`),
                    path: path_1.default.join(this.path, 'node_modules', key),
                });
            }
        }
        // Also add all local miniapps
        const localMiniApps = this.config.miniApps.filter((m) => m.isFilePath);
        for (const m of localMiniApps) {
            const pJson = await (0, ern_core_1.readPackageJson)(m.basePath);
            result.push({
                name: pJson.name,
                packagePath: m,
                path: m.basePath,
            });
        }
        return Promise.resolve(result);
    }
    /**
     * Get all native dependencies packages found in this Composite
     */
    async getNativeDependencies({ manifestId, } = {}) {
        if (this.cachedNativeDependencies) {
            return Promise.resolve(this.cachedNativeDependencies);
        }
        const localMiniApps = this.config.miniApps.filter((m) => m.isFilePath);
        const nativeDependencies = await (0, ern_core_1.findNativeDependencies)([
            this.path,
            ...localMiniApps.map((m) => path_1.default.join(m.basePath, 'node_modules')),
        ], {
            manifestId,
        });
        // Filter out MiniApps that can be falsy considered as native dependencies
        // if developer(s) forgot to npm ignore the android/ios directory
        const miniAppsPackages = await this.getMiniAppsPackages();
        const miniAppsPaths = miniAppsPackages.map((p) => p.path);
        nativeDependencies.all = nativeDependencies.all.filter((x) => !miniAppsPaths.includes(x.basePath));
        nativeDependencies.thirdPartyNotInManifest =
            nativeDependencies.thirdPartyNotInManifest.filter((x) => !miniAppsPaths.includes(x.basePath));
        this.cachedNativeDependencies = nativeDependencies;
        return this.cachedNativeDependencies;
    }
    getReactNativeVersion() {
        return this.packageJson.dependencies['react-native'];
    }
}
exports.GeneratedComposite = GeneratedComposite;
//# sourceMappingURL=GeneratedComposite.js.map