"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBabelRc = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const uuid_1 = require("uuid");
const semver_1 = __importDefault(require("semver"));
const ern_core_1 = require("ern-core");
const getNodeModuleVersion_1 = require("./getNodeModuleVersion");
async function createBabelRc({ cwd, extraPaths = [], }) {
    ern_core_1.log.debug('Creating top level composite .babelrc');
    const compositePackageJson = await (0, ern_core_1.readPackageJson)(cwd);
    const compositeNodeModulesPath = path_1.default.join(cwd, 'node_modules');
    const compositeReactNativeVersion = await (0, getNodeModuleVersion_1.getNodeModuleVersion)({
        cwd,
        name: 'react-native',
    });
    const compositeBabelRc = {
        plugins: [],
    };
    const paths = [
        ...Object.keys(compositePackageJson.dependencies).map((d) => path_1.default.join(compositeNodeModulesPath, d)),
        ...extraPaths,
    ];
    // Ugly hacky way of handling module-resolver babel plugin
    // At least it has some guarantees to make it safer but its just a temporary
    // solution until we figure out a more proper way of handling this plugin
    ern_core_1.log.debug('Taking care of potential Babel module-resolver plugins used by MiniApps');
    let moduleResolverAliases = {};
    for (const p of paths) {
        let miniAppPackageJson;
        try {
            miniAppPackageJson = await (0, ern_core_1.readPackageJson)(p);
        }
        catch (e) {
            // swallow (for test. to be fixed)
            continue;
        }
        const miniAppName = miniAppPackageJson.name;
        if (miniAppPackageJson.babel) {
            if (miniAppPackageJson.babel.plugins) {
                for (const babelPlugin of miniAppPackageJson.babel.plugins) {
                    if (Array.isArray(babelPlugin)) {
                        if (babelPlugin.includes('module-resolver')) {
                            // Add unique name to this composite top level module-resolver to avoid
                            // it messing with other module-resolver plugin configurations that could
                            // be defined in the .babelrc config of individual MiniApps
                            // https://babeljs.io/docs/en/options#plugin-preset-merging
                            babelPlugin.push((0, uuid_1.v4)());
                            // Copy over module-resolver plugin & config to top level composite .babelrc
                            ern_core_1.log.debug(`Taking care of module-resolver Babel plugin for ${miniAppName} MiniApp`);
                            if (compositeBabelRc.plugins.length === 0) {
                                // First MiniApp to add module-resolver plugin & config
                                // easy enough, we just copy over the plugin & config
                                compositeBabelRc.plugins.push(babelPlugin);
                                for (const x of babelPlugin) {
                                    if (x instanceof Object && x.alias) {
                                        moduleResolverAliases = x.alias;
                                        break;
                                    }
                                }
                            }
                            else {
                                // Another MiniApp  has already declared module-resolver
                                // plugin & config. If we have conflicts for aliases, we'll just abort
                                // bundling as of now to avoid generating a potentially unstable bundle
                                for (const item of babelPlugin) {
                                    if (item instanceof Object && item.alias) {
                                        for (const aliasKey of Object.keys(item.alias)) {
                                            if (moduleResolverAliases[aliasKey] &&
                                                moduleResolverAliases[aliasKey] !== item.alias[aliasKey]) {
                                                throw new Error('Babel module-resolver alias conflict');
                                            }
                                            else if (!moduleResolverAliases[aliasKey]) {
                                                moduleResolverAliases[aliasKey] = item.alias[aliasKey];
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else {
                            ern_core_1.log.warn(`Unsupported Babel plugin type ${babelPlugin.toString()} in ${miniAppName} MiniApp`);
                        }
                    }
                    else {
                        ern_core_1.log.warn(`Unsupported Babel plugin type ${babelPlugin.toString()} in ${miniAppName} MiniApp`);
                    }
                }
            }
            ern_core_1.log.debug(`Removing babel object from ${miniAppName} MiniApp package.json`);
            delete miniAppPackageJson.babel;
            await (0, ern_core_1.writePackageJson)(p, miniAppPackageJson);
        }
    }
    if (semver_1.default.gte(compositeReactNativeVersion, '0.57.0')) {
        compositeBabelRc.presets = ['module:metro-react-native-babel-preset'];
    }
    else {
        compositeBabelRc.presets = ['react-native'];
    }
    return fs_extra_1.default.writeFile(path_1.default.join(cwd, '.babelrc'), JSON.stringify(compositeBabelRc, null, 2));
}
exports.createBabelRc = createBabelRc;
//# sourceMappingURL=createBabelRc.js.map