"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateComposite = void 0;
const ern_core_1 = require("ern-core");
const cleanupCompositeDir_1 = require("./cleanupCompositeDir");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const semver_1 = __importDefault(require("semver"));
const lodash_1 = __importDefault(require("lodash"));
const uuid_1 = require("uuid");
const addRNDepToPjson_1 = require("./addRNDepToPjson");
const getNodeModuleVersion_1 = require("./getNodeModuleVersion");
const addRNStartScriptToPjson_1 = require("./addRNStartScriptToPjson");
const createIndexJs_1 = require("./createIndexJs");
const createBaseCompositeImports_1 = require("./createBaseCompositeImports");
const patchCompositeBabelRcRoots_1 = require("./patchCompositeBabelRcRoots");
const patchMetro51AssetsBug_1 = require("./patchMetro51AssetsBug");
const patchMetroAssetsBug_1 = require("./patchMetroAssetsBug");
const patchMetroBabelEnv_1 = require("./patchMetroBabelEnv");
const createBabelRc_1 = require("./createBabelRc");
const applyYarnResolutions_1 = require("./applyYarnResolutions");
const createMetroConfig_1 = require("./createMetroConfig");
const createRNCliConfig_1 = require("./createRNCliConfig");
const installPackages_1 = require("./installPackages");
const installPackagesWithoutYarnLock_1 = require("./installPackagesWithoutYarnLock");
const installExtraPackages_1 = require("./installExtraPackages");
const createWatchmanConfig_1 = require("./createWatchmanConfig");
const cli_table_1 = __importDefault(require("cli-table"));
const os_1 = __importDefault(require("os"));
async function generateComposite(config) {
    var _a;
    ern_core_1.log.debug(`generateComposite config : ${JSON.stringify(config, null, 2)}`);
    // Set env var ERN_BUGSNAG_CODE_BUNDLE_ID as a unique code bundle id for bugsnag
    process.env.ERN_BUGSNAG_CODE_BUNDLE_ID =
        (_a = process.env.ERN_BUGSNAG_CODE_BUNDLE_ID) !== null && _a !== void 0 ? _a : (0, uuid_1.v4)();
    if (config.miniApps.length === 0 &&
        (config.jsApiImplDependencies || []).length === 0) {
        throw new Error(`At least one MiniApp or JS API implementation is needed to generate a composite`);
    }
    return config.baseComposite
        ? generateCompositeFromBase(config.miniApps, config.outDir, config.baseComposite, {
            extraJsDependencies: config.extraJsDependencies,
            jsApiImplDependencies: config.jsApiImplDependencies,
        })
        : generateFullComposite(config.miniApps, config.outDir, {
            extraJsDependencies: config.extraJsDependencies,
            jsApiImplDependencies: config.jsApiImplDependencies,
            metroExtraNodeModules: config.metroExtraNodeModules,
            pathToYarnLock: config.pathToYarnLock,
            resolutions: config.resolutions,
        });
}
exports.generateComposite = generateComposite;
async function generateCompositeFromBase(miniApps, outDir, baseComposite, { extraJsDependencies = [], jsApiImplDependencies, } = {}) {
    if (baseComposite.isRegistryPath) {
        throw new Error(`baseComposite can only be a file or git path (${baseComposite})`);
    }
    if ((await fs_extra_1.default.pathExists(outDir)) && (await fs_extra_1.default.readdir(outDir)).length > 0) {
        throw new Error(`${outDir} directory exists and is not empty.
Composite output directory should either not exist (it will be created) or should be empty.`);
    }
    else {
        ern_core_1.shell.mkdir('-p', outDir);
    }
    if (baseComposite.isGitPath) {
        await (0, ern_core_1.gitCli)().clone(baseComposite.basePath, outDir);
        if (baseComposite.version) {
            await (0, ern_core_1.gitCli)(outDir).checkout(baseComposite.version);
        }
    }
    else {
        ern_core_1.shell.cp('-Rf', path_1.default.join(baseComposite.basePath, '{.*,*}'), outDir);
    }
    const jsPackages = jsApiImplDependencies
        ? [...miniApps, ...jsApiImplDependencies]
        : miniApps;
    ern_core_1.shell.pushd(outDir);
    try {
        await (0, installPackagesWithoutYarnLock_1.installPackagesWithoutYarnLock)({ cwd: outDir, jsPackages });
        await (0, createBaseCompositeImports_1.createBaseCompositeImports)({ cwd: outDir });
        if (extraJsDependencies) {
            await (0, installExtraPackages_1.installExtraPackages)({ cwd: outDir, extraJsDependencies });
        }
    }
    finally {
        ern_core_1.shell.popd();
    }
}
async function generateFullComposite(miniApps, outDir, { extraJsDependencies = [], jsApiImplDependencies = [], pathToYarnLock, resolutions, metroExtraNodeModules, } = {}) {
    if (await fs_extra_1.default.pathExists(outDir)) {
        await ern_core_1.kax
            .task('Cleaning up existing composite directory')
            .run((0, cleanupCompositeDir_1.cleanupCompositeDir)(outDir));
    }
    else {
        ern_core_1.shell.mkdir('-p', outDir);
    }
    ern_core_1.shell.pushd(outDir);
    const remoteMiniapps = miniApps.filter((p) => !p.isFilePath);
    const localMiniApps = miniApps.filter((p) => p.isFilePath);
    const localMiniAppsPaths = localMiniApps.map((m) => m.basePath);
    // Explicitly add react as an extra node module in metro config
    // Because it's not a native module, it will not be auto added
    // later on as done for native modules
    const extraNodeModules = {
        react: path_1.default.join(outDir, 'node_modules/react'),
    };
    try {
        if (remoteMiniapps.length > 0) {
            // Only install remote miniapps coming from git/npm
            await (0, installPackages_1.installPackages)({
                cwd: outDir,
                jsApiImplDependencies,
                miniApps: remoteMiniapps,
                pathToYarnLock,
            });
        }
        else {
            await ern_core_1.yarn.init();
            // We need to install react-native in top level composite as it won't
            // transitively come with install of a miniapp in composite (we didn't
            // `yarn add` any miniapps as they are all local).
            // To know the version to install, we will just have a peak to one of
            // the miniapps, given that react native version is aligned across all.
            const pJson = await (0, ern_core_1.readPackageJson)(localMiniAppsPaths[0]);
            const miniAppRnVersion = pJson.dependencies['react-native'];
            extraJsDependencies.push(ern_core_1.PackagePath.fromString(`react-native@${miniAppRnVersion}`));
            // We also need to keep react in the composite project root as
            // keeping it outside the root will lead to isses with versions of
            // react native <= 0.60.0 but also cause some side effect with
            // more recent react native version (one we identified has to do
            // with react hooks causing a red screen if react is not part of
            // the project root)
            const miniAppReactVersion = pJson.dependencies.react;
            extraJsDependencies.push(ern_core_1.PackagePath.fromString(`react@${miniAppReactVersion}`));
            // Also add latest version of the bridge
            extraJsDependencies.push(ern_core_1.PackagePath.fromString('react-native-electrode-bridge@^1.0.0'));
            extraJsDependencies = [...extraJsDependencies, ...jsApiImplDependencies];
        }
        await (0, addRNStartScriptToPjson_1.addRNStartScriptToPjson)({ cwd: outDir });
        await (0, createIndexJs_1.createIndexJs)({
            cwd: outDir,
            jsApiImplDependencies,
            miniApps,
        });
        await (0, createWatchmanConfig_1.createWatchmanConfig)({ cwd: outDir });
        await ern_core_1.kax.task('Adding extra packages to the composite').run((0, installExtraPackages_1.installExtraPackages)({
            cwd: outDir,
            extraJsDependencies: [
                ern_core_1.PackagePath.fromString('ern-bundle-store-metro-asset-plugin@^1.0.0'),
                ern_core_1.PackagePath.fromString('react-native-svg-transformer@^1.0.0'),
                ...extraJsDependencies,
            ],
        }));
        if (resolutions) {
            // This function should be be called prior to applying
            // any file patches in node_modules, as it will run
            // `yarn install`, thus potentially clearing any previously
            // applied patches
            await (0, applyYarnResolutions_1.applyYarnResolutions)({ cwd: outDir, resolutions });
        }
        let blacklistRe = [];
        if (localMiniApps.length > 0) {
            // If we have some local miniapps we need to do a few extra things.
            // Basically, we need to identify all native modules used by the local
            // miniapps, and build proper blacklistRe / extraNodeModules metro
            // config values.
            // What we want here, is for extraNodeModules to contain a local path
            // to each native module, and blacklist all other existing paths to
            // the native module to avoid duplication conflicts.
            const localMiniAppsNodeModulePaths = localMiniAppsPaths.map((p) => path_1.default.join(p, 'node_modules'));
            const allNativeDeps = await (0, ern_core_1.findNativeDependencies)([
                path_1.default.join(outDir, 'node_modules'),
                ...localMiniAppsNodeModulePaths,
            ]);
            // Exclude api/api impls as they are not native modules
            allNativeDeps.apis = [];
            allNativeDeps.nativeApisImpl = [];
            const dedupedNativeModules = ern_core_1.nativeDepenciesVersionResolution.resolveNativeDependenciesVersionsEx(allNativeDeps);
            if (dedupedNativeModules.pluginsWithMismatchingVersions.length > 0) {
                let errorMsg = `Mismatching native module versions detected.
Native module(s) versions should be aligned across miniapps.
You should resolve the following version mismatches prior to retrying.${os_1.default.EOL}`;
                for (const pkgName of dedupedNativeModules.pluginsWithMismatchingVersions) {
                    const mismatchingPkgs = allNativeDeps.all.filter((x) => x.name === pkgName);
                    const table = new cli_table_1.default({
                        head: ['path', 'version'],
                    });
                    mismatchingPkgs.forEach((pkg) => table.push([pkg.fullPath, pkg.version]));
                    errorMsg += `${table.toString()}${os_1.default.EOL}`;
                }
                throw new Error(errorMsg);
            }
            const allNativeModules = [
                ...allNativeDeps.thirdPartyInManifest,
                ...allNativeDeps.thirdPartyNotInManifest,
            ];
            dedupedNativeModules.resolved.forEach((m) => {
                extraNodeModules[m.name] = m.basePath;
            });
            blacklistRe = lodash_1.default.difference(allNativeModules.map((d) => d.basePath), dedupedNativeModules.resolved.map((d) => d.basePath))
                .concat(...localMiniAppsPaths.map((p) => path_1.default.join(p, 'node_modules/react')))
                .map((l) => new RegExp(os_1.default.platform() === 'win32'
                ? `${l}\\.*`.replace(/\\/g, '\\\\')
                : `${l}\/.*`));
        }
        if (metroExtraNodeModules) {
            Object.keys(metroExtraNodeModules).map((value) => {
                const moduleValue = metroExtraNodeModules[value];
                extraNodeModules[value] = path_1.default.isAbsolute(moduleValue)
                    ? moduleValue
                    : path_1.default.join(outDir, 'node_modules', moduleValue);
            });
        }
        await (0, patchCompositeBabelRcRoots_1.patchCompositeBabelRcRoots)({
            cwd: outDir,
            extraPaths: localMiniAppsPaths,
        });
        await (0, createBabelRc_1.createBabelRc)({ cwd: outDir, extraPaths: localMiniAppsPaths });
        const rnVersion = await (0, getNodeModuleVersion_1.getNodeModuleVersion)({
            cwd: outDir,
            name: 'react-native',
        });
        await (0, createMetroConfig_1.createMetroConfig)({
            blacklistRe,
            cwd: outDir,
            extraNodeModules,
            reactNativeVersion: rnVersion,
            watchFolders: localMiniAppsPaths,
        });
        if (semver_1.default.gte(rnVersion, '0.57.0')) {
            await (0, createRNCliConfig_1.createRNCliConfig)({ cwd: outDir });
        }
        await (0, addRNDepToPjson_1.addRNDepToPjson)(outDir, rnVersion);
        if (semver_1.default.lt(rnVersion, '0.60.0')) {
            await (0, patchMetro51AssetsBug_1.patchMetro51AssetsBug)({ cwd: outDir });
        }
        if (localMiniApps.length > 0) {
            // We only need to apply this patch if there is at least one local
            // MiniApp, as the bug it fixes only happens when dealing with
            // a monorepo like configuration. Such a configuration is only created
            // when at least one local MiniApp is present in the composite
            await (0, patchMetroAssetsBug_1.patchMetroAssetsBug)({ cwd: outDir });
        }
        await (0, patchMetroBabelEnv_1.patchMetroBabelEnv)({ cwd: outDir });
    }
    finally {
        ern_core_1.shell.popd();
    }
}
//# sourceMappingURL=generateComposite.js.map