"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runYarnUsingMiniAppDeltas = exports.getPackageJsonDependenciesUsingMiniAppDeltas = exports.getMiniAppsDeltas = void 0;
const ern_core_1 = require("ern-core");
const yarnLockUtils_1 = require("./yarnLockUtils");
const lodash_1 = __importDefault(require("lodash"));
/**
 *  Using a yarn.lock file content as reference to figure out deltas, group the MiniApps as follow :
 * - 'new' : The MiniApp is a new one (it was not part of previously generated composite)
 * - 'same' : The MiniApp is the same (it was part of previously generated composite, with same version)
 * - 'upgraded' : The MiniApp has a new version (it was part of previously generated composite, but with a different version)
 */
function getMiniAppsDeltas(miniApps, yarnlock) {
    return lodash_1.default.groupBy(miniApps, (m) => {
        const re = (0, yarnLockUtils_1.getYarnLockTopLevelDependencyRe)(m);
        const match = re.exec(yarnlock);
        if (match === null) {
            return 'new';
        }
        else {
            return match[2 /*version*/] === m.version ? 'same' : 'upgraded';
        }
    });
}
exports.getMiniAppsDeltas = getMiniAppsDeltas;
/**
 * Generate package.json dependencies object based on MiniApp deltas.
 * The object will contain the same MiniApps at the same versions that were
 * used to generate the provided yarn.lock, so that we get back to original state
 * conforming with yarn.lock
 * It only contains 'same' and 'upgrade' MiniApps (not new ones)
 */
function getPackageJsonDependenciesUsingMiniAppDeltas(miniAppsDeltas, yarnlock) {
    const result = {};
    if (miniAppsDeltas.same) {
        for (const m of miniAppsDeltas.same) {
            if (m.isRegistryPath) {
                // Sample package.json entry :
                // "test-miniapp": "0.8.3"
                result[m.basePath] = m.version;
            }
            else if (m.isGitPath) {
                // For a git based dependency, the name of the dependency as
                // seen in package.json is not know by the PackagePath object
                // Only way to find the name of the dependency is to look in
                // the yarn.lock file as it records the name of git based dependencies
                const name = (0, yarnLockUtils_1.getYarnLockTopLevelGitDependencyRe)(m).exec(yarnlock)[1 /*name*/];
                // Sample package.json entry :
                // "test-miniapp": "https://github.com/org/test-miniapp.git#master"
                result[name] = m.fullPath;
            }
        }
    }
    if (miniAppsDeltas.upgraded) {
        for (const m of miniAppsDeltas.upgraded) {
            const re = (0, yarnLockUtils_1.getYarnLockTopLevelDependencyRe)(m);
            const initialVersion = re.exec(yarnlock)[2 /*version*/];
            // Please see comment above, in miniAppsDeltas.same to understand
            // the distinction between registry v.s git dependency.
            if (m.isRegistryPath) {
                result[m.basePath] = initialVersion;
            }
            else if (m.isGitPath) {
                const name = (0, yarnLockUtils_1.getYarnLockTopLevelGitDependencyRe)(m).exec(yarnlock)[1 /*name*/];
                result[name] = `${m.basePath}#${initialVersion}`;
            }
        }
    }
    return result;
}
exports.getPackageJsonDependenciesUsingMiniAppDeltas = getPackageJsonDependenciesUsingMiniAppDeltas;
async function runYarnUsingMiniAppDeltas(miniAppsDeltas) {
    //
    // Now we can `yarn add` for new MiniApps
    if (miniAppsDeltas.new) {
        for (const newMiniAppVersion of miniAppsDeltas.new) {
            ern_core_1.log.debug(`Adding new MiniApp ${newMiniAppVersion.toString()}`);
            await ern_core_1.kax
                .task(`Adding ${newMiniAppVersion}`)
                .run(ern_core_1.yarn.add(newMiniAppVersion));
        }
    }
    //
    // We also run `yarn add` for any upgraded MiniApps insead of
    // `yarn upgrade`, to ensure that the dependency graph of the
    // MiniApp will remain as close to the existing yarn.lock as
    // possible
    if (miniAppsDeltas.upgraded) {
        for (const upgradedMiniAppVersion of miniAppsDeltas.upgraded) {
            ern_core_1.log.debug(`Upgrading MiniApp ${upgradedMiniAppVersion.toString()}`);
            await ern_core_1.kax
                .task(`Upgrading ${upgradedMiniAppVersion}`)
                .run(ern_core_1.yarn.add(upgradedMiniAppVersion));
        }
    }
    //
    // If the MiniApp is at the same version, we don't really need
    // to run `yarn add`. There is one exception to this rule however,
    // if the MiniApp package path is git based and is tracking a branch,
    // we want to run `yarn add` even if the branch is the same.
    // While tags or SHAs are precise immutable versions, branches however
    // are mutable, and even if the branch name hasn't changed, the code
    // on the branch might have. That is why we run `yarn add` in this case.
    if (miniAppsDeltas.same) {
        for (const sameMiniAppVersion of miniAppsDeltas.same) {
            if (sameMiniAppVersion.isGitPath &&
                (await ern_core_1.utils.isGitBranch(sameMiniAppVersion))) {
                ern_core_1.log.debug(`Re-adding git based MiniApp ${sameMiniAppVersion.toString()}`);
                await ern_core_1.kax
                    .task(`Adding ${sameMiniAppVersion}`)
                    .run(ern_core_1.yarn.add(sameMiniAppVersion));
            }
        }
    }
}
exports.runYarnUsingMiniAppDeltas = runYarnUsingMiniAppDeltas;
//# sourceMappingURL=miniAppsDeltasUtils.js.map